package  
{
	import Box2D.Collision.Shapes.b2PolygonShape;
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2BodyDef;
	import Box2D.Dynamics.b2World;
	import Box2D.Dynamics.Joints.b2JointDef;
	import Box2D.Dynamics.Joints.b2MouseJoint;
	import Box2D.Dynamics.Joints.b2MouseJointDef;
	import Box2D.Dynamics.Joints.b2RevoluteJoint;
	import Box2D.Dynamics.Joints.b2RevoluteJointDef;
	
	import org.flixel.FlxGroup;
	import org.flixel.FlxG;
	import org.flixel.FlxRect;
	import org.flixel.FlxSprite;
	/**
	 * ...
	 * @author Marek Kapolka
	 */
	public class PhysArm extends FlxGroup
	{
		[Embed(source = "../res/arm.xml", mimeType="application/octet-stream")]
		public static var arm_xml : Class;
		
		[Embed(source = "../res/arm_sleeve.png")]
		public static var arm_sleeve_png : Class;
		
		[Embed(source = "../res/arm_arm.png")]
		public static var arm_arm_png : Class;
		
		[Embed(source = "../res/arm_hand.png")]
		public static var arm_hand_png : Class;
		
		private var _sleeve : FlxPhysBody;
		private var _arm : FlxPhysBody;
		private var _hand : FlxPhysBody;
		
		private var _angle : Number;
		
		private var _world : b2World;
		
		public function PhysArm(ix : Number, iy : Number, w : b2World) 
		{
			super();
			x = ix;
			y = iy;
			
			
			
			_world = w;
			
			loadXML(new XML(new arm_xml()));
			
			reset(x, y);
		}
		
		private function loadXML(xml : XML):void
		{
			var oSleeve : Object = new Object();
			var oArm : Object = new Object();
			var oHand : Object = new Object();
			var oElbow : Object = new Object();
			var oWrist : Object = new Object();
			
			for each (var xc : XML in xml.descendants(new QName("MULE","displayObject")))
			{
				var n : String = xc.attribute("id");
				var o : Object = null;
				
				switch (n)
				{
					case "sleeve":
						o = oSleeve;
					break;
					
					case "arm":
						o = oArm;
					break;
					
					case "hand":
						o = oHand;
					break;
					
					case "sleeve_arm_joint":
						o = oElbow;
					break;
					
					case "wrist_joint":
						o = oWrist;
					break;
				}
				
				o.x = (Number)(xc.x);
				o.y = (Number)(xc.y);
				o.rotation = (Number)(xc.rotation);
				
				trace(n);
				trace(o.x + " " + o.y + " " + o.rotation);
			}
			
			var w : b2World = _world;
			_arm = new FlxRectPhysBody(x + oArm.x, oArm.y * -1, arm_arm_png, w);
			_arm.rotation = oArm.rotation * -1;
			add(_arm);
			
			_hand = new FlxRectPhysBody(x + oHand.x, oHand.y * -1, arm_hand_png, w);
			_hand.rotation = oHand.rotation * -1;
			add(_hand);
			
			_sleeve = new FlxRectPhysBody(x + oSleeve.x, oSleeve.y * -1, arm_sleeve_png, w);
			_sleeve.rotation = oSleeve.rotation * -1;
			add(_sleeve);
			
			
			
			
			//Joints
			var jd : b2RevoluteJointDef = new b2RevoluteJointDef();
			jd.collideConnected = false;
			jd.lowerAngle = Math.PI / 30;
			jd.upperAngle = 3 * Math.PI / 4;
			jd.enableLimit = true;
			jd.referenceAngle = 0;
			
			jd.Initialize(_sleeve.body, _arm.body, new b2Vec2(oElbow.x, oElbow.y * -1));
			w.CreateJoint(jd)
			
			jd.lowerAngle = -Math.PI / 5;
			jd.upperAngle = Math.PI / 5;
			jd.Initialize(_hand.body, _arm.body, new b2Vec2(oWrist.x, oWrist.y * -1));
			w.CreateJoint(jd);
			
			jd.enableLimit = false;
			jd.Initialize(_sleeve.body, w.GetGroundBody(), new b2Vec2(_sleeve.body.GetPosition().x - _sleeve.width / 2, _sleeve.body.GetPosition().y));
			w.CreateJoint(jd);
			
		}
		
		override public function update():void
		{
			super.update();
			
			if (FlxG.mouse.justPressed())
			{
				_angle = _hand.body.GetAngle();
			}
			
			if (FlxG.mouse.pressed())
			{
				//_mouseJoint.SetTarget(new b2Vec2(FlxG.mouse.x, FlxG.mouse.y * -1));
				_hand.body.SetPosition(new b2Vec2(FlxG.mouse.x, FlxG.mouse.y * -1));
				
				_hand.body.SetAngle(_angle);
				
				var dx : Number = _hand.body.GetPosition().x - FlxG.mouse.x;
				var dy : Number = _hand.body.GetPosition().y - FlxG.mouse.y * -1;
				
				//_hand.body.GetLinearVelocity().x = -dx;
				//_hand.body.GetLinearVelocity().y = -dy;
				//_hand.body.ApplyForce(new b2Vec2( -dx * 100, -dy * 100), new b2Vec2(FlxG.mouse.x, -FlxG.mouse.y));
			}
			

		}
		
	}

}