package scene_1 
{
	import linear_texture.LinearTexture;
	import org.flixel.FlxGroup;
	import org.flixel.FlxPoint;
	import org.flixel.FlxSprite;
	import org.flixel.FlxG;
	
	/**
	 * ...
	 * @author Marek Kapolka
	 */
	public class BikePump extends FlxGroup implements Grabable
	{
		[Embed(source = "../../res/scene_1/bike_pump.png")]
		public static var bike_pump_body_png : Class;
		
		[Embed(source = "../../res/scene_1/bike_pump_handle.png")]
		public static var bike_pump_handle_png : Class;
		
		private var _pumpBody : FlxSprite;
		private var _pumpHandle : FlxSprite;
		
		private var _minPosition : FlxPoint = new FlxPoint();
		private var _maxPosition : FlxPoint = new FlxPoint();
		private var _slope : Number = 0;
		
		private var _handOffset : FlxPoint = new FlxPoint();
		
		private var _lungs : Lungs;
		private var _lastV : Number = 0;
		
		private var _curPosition : Number = 0;
		
		private var _pumpTexture : BikePumpTexture;
		
		public function BikePump() 
		{
			super();
			
			_pumpBody = new FlxSprite(0, 0, bike_pump_body_png);
			_pumpHandle = new FlxSprite(0, 0, bike_pump_handle_png);
			
			_pumpBody.offset.x = _pumpBody.width / 2;
			_pumpBody.offset.y = _pumpBody.height / 2;
			
			_pumpHandle.offset.x = _pumpHandle.width / 2;
			_pumpHandle.offset.y = _pumpHandle.height / 2;
			
			add(_pumpHandle);
			add(_pumpBody);
			
			_pumpTexture = new BikePumpTexture();
		}
		
		public function set lungs(l : Lungs):void
		{
			_lungs = l;
		}
		
		public function get lungs():Lungs
		{
			return _lungs;
		}
		
		public function movePumpBody(nx : Number, ny : Number):void
		{
			_pumpBody.x = nx;
			_pumpBody.y = ny;
		}
		
		public function movePumpHandle(nx : Number, ny : Number):void
		{
			_pumpHandle.x = nx;
			_pumpHandle.y = ny;
		}
		
		public function linearMovePumpHandle(v : Number):void
		{	
			if (v > 1) v = 1;
			if (v < 0) v = 0;
			
			var dx : Number = _maxPosition.x - _minPosition.x;
			var dy : Number = _maxPosition.y - _minPosition.y;
			
			var x : Number = _minPosition.x + (v * dx);
			var y : Number =  _minPosition.y + (v * dy);
			
			_pumpHandle.x = x;
			_pumpHandle.y = y;
		}
		
		public function movePumpHandleByY(ny : Number):void
		{
			/*if (_slope != 0)
			{
				var s : Number = 1 / _slope;
				
				_pumpHandle.x = _minPosition.x + (s * (ny - _minPosition.y));
				_pumpHandle.y = ny;
			} else {
				_pumpHandle.y = ny;
			}*/		
			
			boundHandle();
			
			var v : Number = (max(_minPosition.y, _maxPosition.y) - ny) / Math.abs(_minPosition.y - _maxPosition.y);
			
			v = _pumpTexture.move(v);
			
			linearMovePumpHandle(v);
			
			var dv : Number = v - _lastV;
			
			if (dv < 0 && v > .03)
			{
				//lungs.pumped += dv * -1;
				//lungs.pumped += (1 - v) * FlxG.elapsed * 2.5 * (1 + (dv / -100));
				
				var v2 : Number = (1 - v) * (1 - v);
				
				lungs.pumped += v2 * FlxG.elapsed * 2;
			}
			
			_lastV = v;
		}
		
		private function min(a : Number, b : Number):Number
		{
			return (a < b?a:b);
		}
		
		private function max(a : Number, b : Number):Number
		{
			return (a > b?a:b);
		}
		
		private function boundHandle():void
		{
			var minx : Number = min(_minPosition.x, _maxPosition.x);
			var miny : Number = min(_minPosition.y, _maxPosition.y);
			var maxx : Number = max(_minPosition.x, _maxPosition.x);
			var maxy : Number = max(_minPosition.y, _maxPosition.y);
			
			if (_pumpHandle.x < minx) _pumpHandle.x = minx;
			if (_pumpHandle.x > maxx) _pumpHandle.x = maxx;
			if (_pumpHandle.y < miny) _pumpHandle.y = miny;
			if (_pumpHandle.y > maxy) _pumpHandle.y = maxy;
		}
		
		public function setMinPoint(p : FlxPoint):void
		{
			_minPosition = p;
			calcSlope();
		}
		
		public function setMaxPoint(p : FlxPoint):void
		{
			_maxPosition = p;
			calcSlope();
		}
		
		public function setPoint(p : FlxPoint):void
		{
			if (_minPosition.x < p.x) _minPosition.x = p.x;
			if (_minPosition.y < p.y) _minPosition.y = p.y;
			
			if (_maxPosition.x > p.x) _maxPosition.x = p.x;
			if (_maxPosition.y > p.y) _maxPosition.y = p.y;
			
			calcSlope();
		}
		
		private function calcSlope():void
		{
			if (_minPosition == null || _maxPosition == null) return;
			
			_slope =  (_maxPosition.y - _minPosition.y) / (_maxPosition.x - _minPosition.x);
		}
		
		public function grabOverlaps(x : Number, y : Number):Boolean
		{
			return (_pumpHandle.overlapsPoint(x, y, true) && !_pumpBody.overlapsPoint(x,y,true));
		}
		
		public function grab():void
		{
			_handOffset.x = _pumpHandle.x - FlxG.mouse.x;
			_handOffset.y = _pumpHandle.y - FlxG.mouse.y;
		}
		
		public function letGo():void
		{
			
		}
		
		public function pull(nx : Number, ny : Number):void
		{
			var dy : Number = _pumpHandle.y - ny;
			
			/*if (dy > 0)
			{
				_curPosition += .01;
			} else {
				_curPosition -= .01;
			}
			
			if (_curPosition > 1) _curPosition = 1;
			if (_curPosition < 0) _curPosition = 0;
			
			linearMovePumpHandle(_curPosition);*/
			
			movePumpHandleByY(ny + _handOffset.y);
		}
		
		public function getPosition():FlxPoint
		{
			return new FlxPoint(_pumpHandle.x, _pumpHandle.y);
		}
		
	}

}