package scene_1 
{
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2World;
	import Box2D.Dynamics.Joints.b2RevoluteJointDef;
	import flash.display.BitmapData;
	import flash.geom.Rectangle;
	import org.flixel.data.FlxKeyboard;
	import org.flixel.FlxGroup;
	import org.flixel.FlxG;
	import org.flixel.FlxSprite;
	
	/**
	 * ...
	 * @author Marek Kapolka
	 */
	public class Body extends FlxGroup 
	{
		[Embed(source = "../../res/scene_1/body.png")]
		public static var body_ghost_png : Class;
		
		[Embed(source = "../../res/scene_1/body_l_arm.png")]
		public static var body_ghost_l_arm_png : Class; 
		
		[Embed(source = "../../res/scene_1/body_r_arm.png")]
		public static var body_ghost_r_arm_png : Class;
		
		[Embed(source = "../../res/scene_1/body_l_leg.png")]
		public static var body_ghost_l_leg_png : Class;
		
		[Embed(source = "../../res/scene_1/body_r_leg.png")]
		public static var body_ghost_r_leg_png : Class;
		
		private var _bodySprite : FlxSprite;
		private var _lArm : FlxRectPhysBody;
		private var _rArm : FlxRectPhysBody;
		private var _lLeg : FlxRectPhysBody;
		private var _rLeg : FlxRectPhysBody;
		private var _world : b2World;
		
		private var _outSprite : FlxDynamicSprite = new FlxDynamicSprite();
		
		public function Body(world : b2World) 
		{
			super();
			
			_world = new b2World(new b2Vec2(), false);
			
			_outSprite.createGraphic(FlxG.width, FlxG.height, 0xFF00FF00, true);
			add(_outSprite);
			
			alpha = 0;
		}
		
		public function get sprite():FlxSprite
		{
			return _outSprite;
		}
		
		public function set alpha(n : Number):void
		{			
			_outSprite.alpha = n;
		}
		
		public function get alpha():Number
		{
			return _outSprite.alpha;
		}
		
		public function jolt():void
		{
			var v : b2Vec2 = new b2Vec2();
			v.x = (Math.random() - .5) * 5000;
			v.y = (Math.random() - .5) * 5000;
			
			_lArm.body.ApplyImpulse(v, new b2Vec2());
			_rArm.body.ApplyImpulse(v, new b2Vec2());
			_lLeg.body.ApplyImpulse(v, new b2Vec2());
			_rLeg.body.ApplyImpulse(v, new b2Vec2());
		}
		
		override public function update():void
		{
			_world.Step(FlxG.elapsed, 10, 10);
			_world.ClearForces();
			
			var v : b2Vec2 = new b2Vec2();
			v.x = (Math.random() - .5) * 500 * alpha;
			v.y = (Math.random() - .5) * 500 * alpha;
			
			var n : b2Vec2 = new b2Vec2();
			
			_lArm.body.ApplyImpulse(v, n);
			_rArm.body.ApplyImpulse(v, n);
			_lLeg.body.ApplyImpulse(v, n);
			_rLeg.body.ApplyImpulse(v, n);
			
			super.update();
		}
		
		private function updateOutSprite():void
		{
			_rect.x = 0;
			_rect.y = 0;
			_rect.width = FlxG.width;
			_rect.height = FlxG.height;
			
			_outSprite.locked = true;
			_outSprite.fill(0x00000000);
			_outSprite.draw(_bodySprite, _bodySprite.x - _bodySprite.offset.x, _bodySprite.y - _bodySprite.offset.y);
			_outSprite.draw(_rArm, _rArm.x - _rArm.offset.x, _rArm.y - _rArm.offset.y);
			_outSprite.draw(_lArm, _lArm.x - _lArm.offset.x, _lArm.y - _lArm.offset.y);
			_outSprite.draw(_lLeg, _lLeg.x - _lLeg.offset.x, _lLeg.y - _lLeg.offset.y);
			
			_outSprite.locked = false;
			_outSprite.draw(_rLeg, _rLeg.x - _rLeg.offset.x, _rLeg.y - _rLeg.offset.y);
		}
		
		override public function render():void
		{
			updateOutSprite();
			
			super.render();
		}
		
		public function loadXML(xml : XML):void
		{
			var xl : XMLLoader = new XMLLoader(xml);
			
			_bodySprite = new FlxSprite(xl.data.body.x, xl.data.body.y, body_ghost_png);
			_bodySprite.offset.x = _bodySprite.width / 2;
			_bodySprite.offset.y = _bodySprite.height / 2;
			
			_lArm = new FlxRectPhysBody(xl.data.arm_l.x, xl.data.arm_l.y * -1, body_ghost_l_arm_png, _world);
			_rArm = new FlxRectPhysBody(xl.data.arm_r.x, xl.data.arm_r.y * -1, body_ghost_r_arm_png, _world);
			_lLeg = new FlxRectPhysBody(xl.data.leg_l.x, xl.data.leg_l.y * -1, body_ghost_l_leg_png, _world);
			_rLeg = new FlxRectPhysBody(xl.data.leg_r.x, xl.data.leg_r.y * -1, body_ghost_r_leg_png, _world);
			
			_lArm.body.SetLinearDamping(10);
			_rArm.body.SetLinearDamping(10);
			_lLeg.body.SetLinearDamping(10);
			_rLeg.body.SetLinearDamping(10);
			
			var rjd : b2RevoluteJointDef = new b2RevoluteJointDef();
			rjd.bodyB = _world.GetGroundBody();
			rjd.enableLimit = true;
			rjd.referenceAngle = 0;
			rjd.upperAngle = Math.PI / 4;
			rjd.lowerAngle = -Math.PI / 4;
			
			var p : b2Vec2 = new b2Vec2();
			p.x = xl.data.shoulder_l.x / FlxPhysBody.WORLD_SCALE;
			p.y = xl.data.shoulder_l.y / -FlxPhysBody.WORLD_SCALE;
			
			rjd.Initialize(_lArm.body, _world.GetGroundBody(), p);
			_world.CreateJoint(rjd);
			
			p.x = xl.data.shoulder_r.x / FlxPhysBody.WORLD_SCALE;
			p.y = xl.data.shoulder_r.y / -FlxPhysBody.WORLD_SCALE;
			
			rjd.Initialize(_rArm.body, _world.GetGroundBody(), p);
			_world.CreateJoint(rjd);
			
			p.x = xl.data.hip_l.x / FlxPhysBody.WORLD_SCALE;
			p.y = xl.data.hip_l.y / -FlxPhysBody.WORLD_SCALE;
			
			rjd.Initialize(_lLeg.body, _world.GetGroundBody(), p);
			_world.CreateJoint(rjd);
			
			p.x = xl.data.hip_r.x / FlxPhysBody.WORLD_SCALE;
			p.y = xl.data.hip_r.y / -FlxPhysBody.WORLD_SCALE;
			
			rjd.Initialize(_rLeg.body, _world.GetGroundBody(),p);
			_world.CreateJoint(rjd);
			
			add(_lArm);
			add(_rArm);
			add(_lLeg);
			add(_rLeg);
			add(_bodySprite);
			
			_lArm.visible = false;
			_rArm.visible = false;
			_lLeg.visible = false;
			_rLeg.visible = false;
			_bodySprite.visible = false;
			
			alpha = 0;
		}
		
	}

}