package scene_1 
{
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2BodyDef;
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2World;
	import Box2D.Dynamics.Joints.b2DistanceJoint;
	import Box2D.Dynamics.Joints.b2DistanceJointDef;
	import flash.display.BitmapData;
	import flash.display.Shape;
	import linear_texture.LinearTexture;
	
	import org.flixel.FlxGroup;
	import org.flixel.FlxG;
	import org.flixel.FlxPoint;
	import org.flixel.FlxRect;
	import org.flixel.FlxSprite;
	
	/**
	 * ...
	 * @author Marek Kapolka
	 */
	public class PullCord extends FlxGroup implements Grabable
	{
		[Embed(source = "../../res/scene_1/pull_cord.png")]
		public static var pull_cord_png : Class;
		
		private static const CORD_MAX_LENGTH : Number = 400;
		private static const CORD_MIN_LENGTH : Number = 20;
		private static const CORD_SHRINK_SPEED : Number = 1000;
		
		private var _handle : FlxRectPhysBody;
		private var _rope : FlxDynamicSprite;
		private var _raster : Shape = new Shape();
		
		private var _constraint : b2DistanceJoint;
		private var _lastLength : Number = 0;
		
		private var _anchor : b2Body;
		private var _anchorPixelLoc : FlxPoint = new FlxPoint();
		private var _world : b2World;
		
		private var _texture : LinearTexture = new PullCordTexture(CORD_MIN_LENGTH, CORD_MAX_LENGTH);
		
		private var _heart : Heart;
		
		public function PullCord(world : b2World, heart : Heart) 
		{
			_world = world;
			_heart = heart;
			
			_rope = new FlxDynamicSprite();
			_rope.createGraphic(FlxG.width, FlxG.height, 0x00000000, true);
			add(_rope);
			
			_handle = new FlxRectPhysBody(0, 0, pull_cord_png,world);
			add(_handle);
			_handle.body.SetLinearDamping(.5);
		}
		
		public function moveHandle(x : Number, y : Number):void
		{
			_handle.setPositionPixels(x, y);// (new b2Vec2(x, y * -1));
		}
		
		public function placeAnchor(nx : Number, ny : Number):void
		{			
			_anchorPixelLoc.x = nx;
			_anchorPixelLoc.y = ny;
			
			nx /= FlxPhysBody.WORLD_SCALE;
			ny /= FlxPhysBody.WORLD_SCALE;
			
			//Anchor body;
			var ad : b2BodyDef = new b2BodyDef();
			ad.position = new b2Vec2(nx, ny * -1);
			ad.type = b2Body.b2_staticBody;
			
			var anchorBody : b2Body = _world.CreateBody(ad);
			_anchor = anchorBody;
			
			//Add the constraint
			var dd : b2DistanceJointDef = new b2DistanceJointDef();
			dd.bodyA = _handle.body;
			dd.bodyB = anchorBody;
			dd.localAnchorA = new b2Vec2(0, 0);
			dd.localAnchorB = new b2Vec2(0, 0);
			
			var dx : Number = _handle.body.GetPosition().x - nx;
			var dy : Number = _handle.body.GetPosition().y - ny * -1;
			
			dd.length = Math.sqrt(dx * dx + dy * dy);
			_lastLength = dd.length;
			
			_constraint = (b2DistanceJoint)(_world.CreateJoint(dd));
			
			//Slight hack to make the heart start without moving
			update();
			_heart.beatVelocity = 0;
		}
		
		public function grabOverlaps(x : Number, y : Number):Boolean
		{
			return _handle.overlapsPoint(x, y);
		}
		
		public function grab():void
		{
			_handle.body.SetAwake(false);
		}
		
		public function letGo():void
		{
			_handle.body.SetAwake(true);
		}
		
		private function distance(a : b2Vec2, b : b2Vec2):Number
		{
			var dx : Number = a.x - b.x;
			var dy : Number = a.y - b.y;
			
			return Math.sqrt(dx * dx + dy * dy);
		}
		
		public function pull(x : Number, y : Number):void
		{
			/*
			//_handle.setPositionPixels(x, y);
			
			var d : Number = distance(_handle.body.GetPosition(), _anchor.GetPosition());
			
			if (d > CORD_MAX_LENGTH) d = CORD_MAX_LENGTH;
			
			_constraint.SetLength(d);
			
			var dx : Number = x - _handle.x;
			var dy : Number = y - _handle.y;
			
			_handle.body.ApplyImpulse(new b2Vec2(dx, dy * -1), new b2Vec2());*/
			var dx : Number = x - _anchorPixelLoc.x;
			var dy : Number = y - _anchorPixelLoc.y;
			
			var dir : Number = Math.atan2(dy, dx);
			
			var d : Number = Math.sqrt(dx * dx + dy * dy);
			
			var d2 : Number = _texture.move(d);
			
			var dd : Number = d2 / d;
			
			var nx : Number = _anchorPixelLoc.x + Math.cos(dir) * d2;
			var ny : Number = _anchorPixelLoc.y + Math.sin(dir) * d2;
			
			_handle.setPositionPixels(nx, ny);
			
			_constraint.SetLength(d2 / FlxPhysBody.WORLD_SCALE);	
		}
		
		public function getPosition():FlxPoint
		{
			return new FlxPoint(_handle.x, _handle.y);
		}
		
		private function updateRope():void
		{
			_raster.graphics.clear();
			_raster.graphics.moveTo(_anchorPixelLoc.x, _anchorPixelLoc.y);
			_raster.graphics.lineStyle(10, 0xFFFFFFFF);
			_raster.graphics.lineTo(_handle.x, _handle.y);
			_rope.pixels.draw(_raster);
			_rope.forceCalcFrame();
		}
		
		private var b : Boolean = false;
		
		override public function update():void
		{
			if (FlxG.stage != null && !b)
			{
				FlxG.stage.addChild(_raster);
				b = true;
			}
			
			super.update();
			
			var dy : Number = _handle.body.GetPosition().y - _anchor.GetPosition().y;
			var dx : Number = _handle.body.GetPosition().x - _anchor.GetPosition().x;
			_handle.body.SetAngle(Math.atan2(dy, dx));
			
			dx = _handle.x - _anchorPixelLoc.x;
			dy = _handle.y - _anchorPixelLoc.y;
			
			//var l : Number = _constraint.GetLength();
			var l : Number = Math.sqrt(dx * dx + dy * dy);
			
			if (l > _lastLength)
			{
				_heart.beatCounter += (l - _lastLength) / 200;
				_heart.beatVelocity += (l - _lastLength) / 50;
			}
			
			_lastLength = l;
			
			l -= FlxG.elapsed * CORD_SHRINK_SPEED;
			
			if (l > CORD_MAX_LENGTH) l = CORD_MAX_LENGTH;
			if (l < CORD_MIN_LENGTH) l = CORD_MIN_LENGTH;
			
			_constraint.SetLength(l / FlxPhysBody.WORLD_SCALE);
			
			
		}
		
	}

}