package scene_1 
{
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2World;
	import org.flixel.FlxPoint;
	import org.flixel.FlxState;
	import org.flixel.FlxG;
	import scene_2.Scene2Intro;
	
	/**
	 * ...
	 * @author Marek Kapolka
	 */
	public class Scene1State extends FlxState 
	{
		[Embed(source = "../../res/scene_1/scene_1.xml", mimeType = "application/octet-stream")]
		public static var scene_1_xml : Class;
		
		private var _world : b2World;
		
		private var _heart : Heart;
		private var _lungs : Lungs;
		private var _stomach : Stomach;
		private var _pump : BikePump;
		private var _cord : PullCord;
		private var _plunger : Plunger;
		private var _body : Body;
		
		private var _lungsStarted : Boolean = false;
		private var _heartStarted : Boolean = false;
		private var _stomachStarted : Boolean = false;
		
		private var _finishing : Boolean = false;
		
		private var _hand : HandCursor;
		
		private var _grabables : Vector.<Grabable>;
		private var _grabbed : Grabable = null;
		private var _grabbedOffset : FlxPoint = new FlxPoint();
		
		public function Scene1State() 
		{
			initWorld();
			
			_grabables = new Vector.<Grabable>();
			
			FlxG.mouse.show();
			
			loadXML();
			
			_hand = new HandCursor();
			add(_hand);
			FlxG.mouse.hide();
		}
		
		private function initWorld():void
		{
			var gravity : b2Vec2 = new b2Vec2(0, -10);
			_world = new b2World(gravity, false);
			FlxPhysBody.WORLD_SCALE = 10;
		}
		
		public function loadXML():void
		{
			var x : XML = new XML(new scene_1_xml());
			
			var displayObjects : XMLList = x.descendants(new QName("MULE", "displayObject"));
			
			var oHeart : Object = new Object();
			var oLungs : Object = new Object();
			var oStomach : Object = new Object();
			var oPump : Object = new Object;
			var oCord : Object = new Object;
			
			for each (var disp : XML in displayObjects)
			{
				var id : String = disp.attribute("id");
				var o : Object = null;
				
				switch (id)
				{
					case "heart":
						o = oHeart;
					break;
					
					case "lungs":
						o = oLungs;
					break;
					
					case "stomach":
						o = oStomach;
					break;
					
					case "bike_pump":
						oPump.px = disp.x;
						oPump.py = disp.y;
					break;
					
					case "bike_pump_handle":
						oPump.hx = disp.x;
						oPump.hy = disp.y;
					break;
					
					case "pull_cord":
						oCord.x = disp.x;
						oCord.y = disp.y;
					break;
					
					case "pull_cord_anchor":
						oCord.ax = disp.x;
						oCord.ay = disp.y;
					break;
					
					case "pump_min_position":
						oPump.minx = disp.x;
						oPump.miny = disp.y;
					break;
					
					case "pump_max_position":
						oPump.maxx = disp.x;
						oPump.maxy = disp.y;
					break;
				}
				
				if (o != null)
				{
					o.x = disp.x;
					o.y = disp.y;
				}
			}
			
			//Make the objects
			_heart = new Heart(oHeart.x, oHeart.y);
			//_stomach = new Stomach(oStomach.x, oStomach.y);
			_lungs = new Lungs(oLungs.x, oLungs.y);
			_pump = new BikePump();
			_cord = new PullCord(_world, _heart);
			
			_pump.movePumpBody(oPump.px, oPump.py);
			_pump.movePumpHandle(oPump.hx, oPump.hy);
			_pump.setMinPoint(new FlxPoint(oPump.minx, oPump.miny));
			_pump.setMaxPoint(new FlxPoint(oPump.maxx, oPump.maxy));
			_pump.lungs = _lungs;
			
			_cord.moveHandle(oCord.x, oCord.y);
			_cord.placeAnchor(oCord.ax, oCord.ay);
			
			var xmll : XMLLoader = new XMLLoader(new XML(new scene_1_xml()));
			
			_body = new Body(_world);
			_body.loadXML(x);
			
			_stomach = new Stomach(oStomach.x, oStomach.y);
			_stomach.loadXML(xmll);
			
			_plunger = new Plunger(_stomach);
			_plunger.loadXML(x);
			
			add(_body);
			add(_stomach);
			add(_pump);
			add(_lungs);
			add(_heart);
			add(_plunger);
			add(_cord);
			
			_grabables.push(_pump);
			_grabables.push(_cord);
			_grabables.push(_plunger);
		}
		
		override public function update():void
		{
			handleGrabbing();
			
			_world.Step(FlxG.elapsed, 10, 10);
			_world.ClearForces();
			
			//Lungs
			if (!_lungsStarted && _lungs.started)
			{
				_lungsStarted = true;
				
				_body.alpha += .2;
				_body.jolt();
			}
			
			//Heart
			if (!_heartStarted && _heart.started)
			{
				_heartStarted = true;
				
				_body.alpha += .2;
				_body.jolt();
			}
			
			//Stomach
			if (!_stomachStarted && _stomach.started)
			{
				_stomachStarted = true;
				
				_body.alpha += .2;
				_body.jolt();
			}
			
			if (_stomachStarted && _heartStarted && _lungsStarted && !_finishing)
			{
				_finishing = true;
				FlxG.fade.start(0xFF000000, 5, finish);
			}
			
			super.update();
		}
		
		private function finish():void
		{
			FlxG.state = new Scene2Intro();
		}
		
		private function handleGrabbing():void
		{
			if (_grabbed == null)
			{
				_hand.play("point");
				
				for each (var g : Grabable in _grabables)
				{
					if (g.grabOverlaps(FlxG.mouse.x, FlxG.mouse.y))
					{
						if (FlxG.mouse.justPressed())
						{
							_hand.play("grab");
							_hand.lock = true;
							_grabbed = g;
							_grabbed.grab();
							
							var p : FlxPoint = _grabbed.getPosition();
							
							_grabbedOffset.x = _hand.hand.x - p.x;
							_grabbedOffset.y = _hand.hand.y - p.y;
						} else {
							_hand.play("hand");
						}
					}
				}
			} else {
				if (FlxG.mouse.pressed())
				{
					_grabbed.pull(FlxG.mouse.x, FlxG.mouse.y);
					
					p = _grabbed.getPosition();
					
					_hand.hand.x = p.x + _grabbedOffset.x;
					_hand.hand.y = p.y + _grabbedOffset.y;
				} else {
					_grabbed.letGo();
					_grabbed = null;
					_hand.play("point");
					_hand.lock = false;
				}
			}
		}
		
	}

}