package scene_2 
{
	import Box2D.Collision.Shapes.b2MassData;
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2FilterData;
	import Box2D.Dynamics.b2Fixture;
	import Box2D.Dynamics.b2World;
	import Box2D.Dynamics.Joints.b2MouseJoint;
	import Box2D.Dynamics.Joints.b2MouseJointDef;
	import Box2D.Dynamics.Joints.b2RevoluteJoint;
	import Box2D.Dynamics.Joints.b2RevoluteJointDef;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import org.flixel.FlxGroup;
	import org.flixel.FlxG;
	import org.flixel.FlxPoint;
	import org.flixel.FlxU;
	import org.flixel.FlxObject;
	import org.flixel.FlxRect;
	
	/**
	 * ...
	 * @author Marek Kapolka
	 */
	public class Body extends FlxGroup 
	{
		[Embed(source = "../../res/scene_2/body.png")]
		public static var body_png : Class;
		
		[Embed(source = "../../res/scene_2/hips.png")]
		public static var hips_png : Class;
		
		[Embed(source = "../../res/scene_2/head.png")]
		public static var head_png : Class;
		
		[Embed(source = "../../res/scene_2/arm_upper_l.png")]
		public static var arm_upper_l_png : Class;
		
		[Embed(source = "../../res/scene_2/arm_lower_l.png")]
		public static var arm_lower_l_png : Class;
		
		[Embed(source = "../../res/scene_2/hand_l.png")]
		public static var hand_l_png : Class;
		
		[Embed(source = "../../res/scene_2/arm_lower_r.png")]
		public static var arm_lower_r_png : Class;
		
		[Embed(source = "../../res/scene_2/arm_upper_r.png")]
		public static var arm_upper_r_png : Class;
		
		[Embed(source = "../../res/scene_2/hand_r.png")]
		public static var hand_r_png : Class;
		
		[Embed(source = "../../res/scene_2/leg_l.png")]
		public static var leg_l_png : Class;
		
		[Embed(source = "../../res/scene_2/leg_lower_l.png")]
		public static var leg_lower_l_png : Class;
		
		[Embed(source = "../../res/scene_2/leg_r.png")]
		public static var leg_r_png : Class;
		
		[Embed(source = "../../res/scene_2/leg_lower_r.png")]
		public static var leg_lower_r_png : Class;
		
		[Embed(source = "../../res/scene_2/foot_l.png")]
		public static var foot_l_png : Class;
		
		[Embed(source = "../../res/scene_2/foot_r.png")]
		public static var foot_r_png : Class;
		
		[Embed(source = "../../res/scene_2/body_layout.xml", mimeType = "application/octet-stream")]
		public static var body_layout_xml : Class;
		
		private var _body : FlxRectPhysBody;
		private var _hips : FlxRectPhysBody;
		private var _head : FlxRectPhysBody;
		
		private var _luarm : FlxRectPhysBody;
		private var _llarm : FlxRectPhysBody;
		private var _lhand : FlxRectPhysBody;
		
		private var _ruarm : FlxRectPhysBody;
		private var _rlarm : FlxRectPhysBody;
		private var _rhand : FlxRectPhysBody;
		
		private var _lleg : FlxRectPhysBody; 
		private var _llleg : FlxRectPhysBody;
		private var _lfoot : FlxRectPhysBody;
		
		private var _rleg : FlxRectPhysBody;
		private var _rlleg : FlxRectPhysBody;
		private var _rfoot : FlxRectPhysBody;
		
		private var _world : b2World;
		
		private var _origin : Point = new Point();
		private var _angle : Number = 0;
		
		private var _grabables : Vector.<FlxRectPhysBody> = new Vector.<FlxRectPhysBody>();
		private var _overlapped : FlxRectPhysBody;
		private var _grabbed : FlxRectPhysBody;
		
		private var _mj : b2MouseJoint;
		
		public function Body(world : b2World, sx : Number, sy : Number, r : Number = 0)
		{
			super();
			
			_origin.x = sx;
			_origin.y = sy;
			
			_angle = r;
			
			_world = world;
			
			_body = new FlxRectPhysBody(0, 0, Body.body_png, world);
			_hips = new FlxRectPhysBody(0, 0, hips_png, world);
			_head = new FlxRectPhysBody(0, 0, head_png, world);
			
			makeImmovable(_body.body);
			makeImmovable(_hips.body);
			
			_luarm = new FlxRectPhysBody(0, 0, arm_upper_l_png, world);
			_llarm = new FlxRectPhysBody(0, 0, arm_lower_l_png, world);
			_lhand = new FlxRectPhysBody(0, 0, hand_l_png, world);
			
			_ruarm = new FlxRectPhysBody(0, 0, arm_upper_r_png, world);
			_rlarm = new FlxRectPhysBody(0, 0, arm_lower_r_png, world);
			_rhand = new FlxRectPhysBody(0, 0, hand_r_png, world);
			
			_lleg = new FlxRectPhysBody(0, 0, leg_l_png, world);
			_llleg = new FlxRectPhysBody(0, 0, leg_lower_l_png, world);
			_lfoot = new FlxRectPhysBody(0, 0, foot_l_png, world);
			
			_rleg = new FlxRectPhysBody(0, 0, leg_r_png, world);
			_rlleg = new FlxRectPhysBody(0, 0, leg_lower_r_png, world);
			_rfoot = new FlxRectPhysBody(0, 0, foot_r_png, world);
			
			add(_lleg);
			add(_llleg);
			add(_lfoot);
			
			add(_rleg);
			add(_rlleg);
			add(_rfoot);
			
			add(_head);
			add(_hips);
			add(_body);
			
			add(_luarm);
			add(_llarm);
			add(_lhand);
			
			add(_ruarm);
			add(_rlarm);
			add(_rhand);
			

			
			loadXML(new XMLLoader(new XML(new body_layout_xml())));
		}
		
		private function makeImmovable(b : b2Body):void
		{
			var md : b2MassData = new b2MassData();
			b.GetMassData(md);
			md.I *= 1000;
			md.mass *= 100;
			b.SetMassData(md);
		}
		
		override public function add(object : FlxObject, ss : Boolean = false):FlxObject
		{
			super.add(object , ss);
			
			if (object is FlxRectPhysBody)
			{
				_grabables.push((FlxRectPhysBody)(object));
				
				var r : FlxRectPhysBody = (FlxRectPhysBody)(object);
				r.body.SetLinearDamping(10);
				
				var fix : b2Fixture = r.body.GetFixtureList();
				
				while (fix != null)
				{
					var fd : b2FilterData = fix.GetFilterData();
					//fd.categoryBits = 0x0002;
					fd.groupIndex = -10;
					//fd.maskBits = 0x0002;
					
					fix = fix.GetNext();
				}
				
				
				r.body.GetFixtureList().SetFilterData(fd);
			}
			
			return object;
		}
		
		public function get hand():FlxRectPhysBody
		{
			return _rhand;
		}
		
		public function get hand_l():FlxRectPhysBody
		{
			return _lhand;
		}
		
		public function get body():FlxRectPhysBody
		{
			return _body;
		}
		
		override public function update():void
		{
			super.update();
		}
		
		public function canGrab(mx : Number, my : Number):Boolean
		{			
			for each (var o : FlxRectPhysBody in _grabables)
			{
				var p : FlxPoint = FlxU.rotatePoint(mx, my, o.x, o.y, o.angle * -1);
				
				if (o.overlapsPoint(p.x, p.y, true))
				{					
					_overlapped = o;
					return true;
				}
			}
			
			return false;
		}
		
		public function grab():FlxRectPhysBody
		{
			_grabbed = _overlapped;
			
			return _grabbed;
		}
		
		public function letGo():void
		{
			_grabbed = null;
			
			if (_mj != null)
			{
				_world.DestroyJoint(_mj);
				_mj = null;
			}
		}
		
		public function loadXML(xml : XMLLoader):void
		{
			var d : Object = xml.data;
			
			var offset : Point = new Point();
			offset.x = d.body.x;
			offset.y = d.body.y;
			
			offsetData(d, offset, _angle);
			
			_body.setPositionPixels(d.body.x, d.body.y);
			_head.setPositionPixels(d.head.x, d.head.y);
			_hips.setPositionPixels(d.hips.x, d.hips.y);
			
			_luarm.setPositionPixels(d.arm_upper_l.x, d.arm_upper_l.y);
			_llarm.setPositionPixels(d.arm_lower_l.x, d.arm_lower_l.y);
			_lhand.setPositionPixels(d.hand_l.x, d.hand_l.y);
			
			_ruarm.setPositionPixels(d.arm_upper_r.x, d.arm_upper_r.y);
			_rlarm.setPositionPixels(d.arm_lower_r.x, d.arm_lower_r.y);
			_rhand.setPositionPixels(d.hand_r.x, d.hand_r.y);
			
			_lleg.setPositionPixels(d.leg_l.x, d.leg_l.y);
			_llleg.setPositionPixels(d.leg_lower_l.x, d.leg_lower_l.y);
			_lfoot.setPositionPixels(d.foot_l.x, d.foot_l.y);
			
			_rleg.setPositionPixels(d.leg_r.x, d.leg_r.y);
			_rlleg.setPositionPixels(d.leg_lower_r.x, d.leg_lower_r.y);
			_rfoot.setPositionPixels(d.foot_r.x, d.foot_r.y);
			
			if (_angle != 0)
			{
				_body.body.SetAngle(d.body.rotation);
				_head.body.SetAngle(d.head.rotation);
				_hips.body.SetAngle(d.hips.rotation);
				
				_luarm.body.SetAngle(d.arm_upper_l.rotation);
				_llarm.body.SetAngle(d.arm_lower_l.rotation);
				_lhand.body.SetAngle(d.hand_l.rotation);
				
				_ruarm.body.SetAngle(d.arm_upper_r.rotation);
				_rlarm.body.SetAngle(d.arm_lower_r.rotation);
				_rhand.body.SetAngle(d.hand_r.rotation);
				
				_lleg.body.SetAngle(d.leg_l.rotation);
				_llleg.body.SetAngle(d.leg_lower_l.rotation);
				_lfoot.body.SetAngle(d.foot_l.rotation);
				
				_rleg.body.SetAngle(d.leg_r.rotation);
				_rlleg.body.SetAngle(d.leg_lower_r.rotation);
				_rfoot.body.SetAngle(d.foot_r.rotation);
			}
			
			var rd : b2RevoluteJointDef = new b2RevoluteJointDef();
			var p : b2Vec2 = new b2Vec2();
			
			//Neck
			makeJoint(_head.body, _body.body, -Math.PI / 20, Math.PI / 20, d.neck);
			//Small of Back
			makeJoint(_body.body, _hips.body, -Math.PI / 10, Math.PI / 10, d.back_joint);
			
			//Left Shoulder
			makeJoint(_body.body, _luarm.body, -Math.PI / 2, Math.PI / 2, d.shoulder_l);
			//Left Elbow
			makeJoint(_luarm.body, _llarm.body, -Math.PI, 0, d.elbow_l);
			//Left Wrist
			makeJoint(_llarm.body, _lhand.body, -Math.PI / 3, Math.PI / 3, d.wrist_l);
			
			//Right Shoulder
			makeJoint(_body.body, _ruarm.body, -Math.PI / 2, Math.PI / 2, d.shoulder_r);
			//Right Elbow
			makeJoint(_ruarm.body, _rlarm.body, 0, Math.PI, d.elbow_r);
			//Left Wrist
			makeJoint(_rlarm.body, _rhand.body, -Math.PI / 3, Math.PI / 3, d.wrist_r);
			
			//Hip Left
			makeJoint(_hips.body, _lleg.body, -Math.PI / 20, Math.PI / 20, d.hip_joint_l);
			//Hip Right
			makeJoint(_hips.body, _rleg.body, -Math.PI / 20, Math.PI / 20, d.hip_joint_r);
			
			//Knee Left
			makeJoint(_lleg.body, _llleg.body, -Math.PI / 10, Math.PI / 10, d.knee_l);
			//Ankle Left
			makeJoint(_llleg.body, _lfoot.body, -Math.PI / 10, Math.PI / 10, d.ankle_l);
			
			//Knee Right
			makeJoint(_rleg.body, _rlleg.body, -Math.PI / 10, Math.PI / 10, d.knee_r);
			//Anke Right
			makeJoint(_rlleg.body, _rfoot.body, -Math.PI / 10, Math.PI / 10, d.ankle_r);
		}
		
		private function offsetData(d : Object, o : Point, r : Number = 0 ):void		
		{
			offsetEntry(d.body, o, r);
			offsetEntry(d.head, o, r);
			offsetEntry(d.hips, o, r);
			
			offsetEntry(d.arm_upper_l, o, r);
			offsetEntry(d.arm_lower_l, o, r);
			offsetEntry(d.hand_l, o, r);
			
			offsetEntry(d.arm_upper_r, o, r);
			offsetEntry(d.arm_lower_r, o, r);
			offsetEntry(d.hand_r, o, r);
			
			offsetEntry(d.leg_l, o, r);
			offsetEntry(d.leg_lower_l, o, r);
			offsetEntry(d.foot_l, o, r);
			
			offsetEntry(d.leg_r, o, r);
			offsetEntry(d.leg_lower_r, o, r);
			offsetEntry(d.foot_r, o, r);
			
			offsetEntry(d.neck, o, r);
			offsetEntry(d.back_joint, o, r);
			offsetEntry(d.hip_joint_l, o, r);
			offsetEntry(d.hip_joint_r, o, r);
			offsetEntry(d.knee_l, o, r);
			offsetEntry(d.knee_r, o, r);
			offsetEntry(d.ankle_l, o, r);
			offsetEntry(d.ankle_r, o, r);
			
			offsetEntry(d.shoulder_l, o, r);
			offsetEntry(d.elbow_l, o, r);
			offsetEntry(d.wrist_l, o, r);
			
			offsetEntry(d.shoulder_r, o, r);
			offsetEntry(d.elbow_r, o, r);
			offsetEntry(d.wrist_r, o, r);
		}
		
		private function offsetEntry(d : Object, o : Point, r : Number = 0):void
		{
			d.x -= o.x;
			d.y -= o.y;
			
			if (r != 0)
			{
				var m : Matrix = new Matrix();
				m.rotate(r);
				var p : Point = m.transformPoint(new Point(d.x, d.y));
				
				d.x = p.x;
				d.y = p.y;
				
				d.rotation = (Number)(d.rotation) - r;
			}
			
			d.x += _origin.x;
			d.y += _origin.y;
		}
		
		private function makeJoint(bodyA : b2Body, bodyB : b2Body, lowerAngle : Number , upperAngle : Number, point : Object):void
		{
			var rd : b2RevoluteJointDef = new b2RevoluteJointDef();
			var p : b2Vec2 = new b2Vec2();
			
			p.x = point.x / FlxPhysBody.WORLD_SCALE;
			p.y = point.y * -1 / FlxPhysBody.WORLD_SCALE;
			
			rd.enableLimit = true;
			rd.collideConnected = false;
			rd.referenceAngle = 0; 0
			rd.upperAngle = upperAngle;
			rd.lowerAngle = lowerAngle;
			
			rd.Initialize(bodyA, bodyB, p);
			_world.CreateJoint(rd);
		}
		
	}

}