package scene_2 
{
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2World;
	import flash.geom.Point;
	import org.flixel.FlxPoint;
	import org.flixel.FlxState;
	import org.flixel.FlxG;
	import org.flixel.FlxU;
	import scene_3.Scene3Intro;
	
	/**
	 * ...
	 * @author Marek Kapolka
	 */
	public class Scene2State extends FlxState 
	{
		[Embed(source = "../../res/scene_2/layout.xml", mimeType = "application/octet-stream")]
		public static var layout_xml : Class;
		
		private var _world : b2World;
		
		private var _body : Body;
		private var _cursor : Cursor;
		private var _background : Background;
		
		private var _grabbing : FlxRectPhysBody;
		private var _offset : Point = new Point;
		
		private var _handLastY : Number;
		private var _yTravelled : Number = 0;
		private var _finishing : Boolean = false;
		
		public function Scene2State() 
		{
			super();
			
			//FlxG.followBounds(0, -1000, FlxG.width, FlxG.height);
			FlxPhysBody.WORLD_SCALE = 1;
			
			_world = new b2World(new b2Vec2(), false);
			
			
			_background = new Background();
			add(_background);
		}
		
		override public function create():void
		{
			//FlxG.follow(_body.body);
			
			loadXML();
		}
		
		override public function update():void
		{
			_world.Step(FlxG.elapsed, 10, 10);
			
			//_body.hand.setPositionPixels(FlxG.mouse.x, FlxG.mouse.y);
			
			if (_body.canGrab(FlxG.mouse.x, FlxG.mouse.y))
			{			
				if (_grabbing == null)
				{
					_cursor.play("open");
				}
				
				if (FlxG.mouse.justPressed())
				{
					_cursor.play("closed");
					_grabbing = _body.grab();
					
					//setup offset
					var p : FlxPoint = FlxU.rotatePoint(FlxG.mouse.x, FlxG.mouse.y, _grabbing.x, _grabbing.y, -_grabbing.angle);
					_offset.x = p.x - _grabbing.x;
					_offset.y = p.y - _grabbing.y;
					
					_cursor.grabbing = true;
				}
			} else {
				if (_grabbing == null)
				{
					_cursor.play("idle");
				}
			}
			
			if (_grabbing != null)
			{
				var rp : FlxPoint = FlxU.rotatePoint(_grabbing.x + _offset.x, _grabbing.y + _offset.y, _grabbing.x, _grabbing.y, _grabbing.angle);
				
				_cursor.setHandPosition(rp.x, rp.y);
				pull(_grabbing, FlxG.mouse.x, FlxG.mouse.y, rp.x, rp.y * -1);
			}
			
			if (FlxG.mouse.justReleased())
			{
				_body.letGo();
				_grabbing = null;
				_cursor.grabbing = false;
			}
			
			//HAND
			
			if (_grabbing != null && _body.hand.y > _handLastY)
			{
				_body.body.body.ApplyImpulse(new b2Vec2(0, 900000), _body.body.body.GetPosition());
				_background.scroll(_body.hand.y - _handLastY);
				_yTravelled += _body.hand.y - _handLastY;
			}
			
			if (_yTravelled > 1500)
			{
				_finishing = true;
				FlxG.fade.start(0xFF000000, 1, finish);
			}
			
			_handLastY = _body.hand.y;
			
			super.update();
		}
		
		private function finish():void
		{
			FlxG.state = new Scene3Intro();
		}
		
		private function pull(b : FlxRectPhysBody, tx : Number, ty : Number, px : Number, py : Number):void
		{
			var dx : Number = tx - b.x;
			var dy : Number = ty - b.y;
			
			dx *= 10000;
			dy *= -10000;
			
			var i : b2Vec2 = new b2Vec2(dx, dy);
			var p : b2Vec2 = new b2Vec2(px, py);
			
			b.body.ApplyImpulse(i, p);
		}
		
		private function stopAllWorldMotion():void
		{
			var body : b2Body = _world.GetBodyList();
			var zero : b2Vec2 = new b2Vec2();
			
			while (body != null)
			{
				body.SetLinearVelocity(zero);
				body.SetAngularVelocity(0);
				
				body = body.GetNext();
			}
		}
		
		private function loadXML():void
		{
			var loader : XMLLoader = new XMLLoader(new XML(new layout_xml()));
			var d : Object = loader.data;
			
			_body = new Body(_world, d.body.x, d.body.y);
			
			_body.hand.body.SetType(b2Body.b2_kinematicBody);
			_body.hand_l.body.SetType(b2Body.b2_kinematicBody);
			
			_body.hand.setPositionPixels(d.hand.x, d.hand.y);
			_body.hand_l.setPositionPixels(d.hand_l.x, d.hand_l.y);
			
			_body.hand.body.SetAngle(d.hand.rotation * -1);
			_body.hand_l.body.SetAngle(d.hand_l.rotation * -1);
			
			_world.Step(1, 10, 10);
			stopAllWorldMotion();
			
			
			_body.hand.body.SetType(b2Body.b2_dynamicBody);
			_body.hand_l.body.SetType(b2Body.b2_dynamicBody);
			
			_cursor = new Cursor();
			
			add(_body);
			add(_cursor);
		}
		
	}

}