package scene_3 
{
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2World;
	import Box2D.Dynamics.b2Body;
	import flash.geom.Point;
	import org.flixel.FlxGroup;
	import org.flixel.FlxG;
	import org.flixel.FlxSprite;
	import org.flixel.FlxState;
	import scene_2.Body;
	
	/**
	 * ...
	 * @author Marek Kapolka
	 */
	public class Scene3State extends FlxState 
	{
		[Embed(source = "../../res/scene_3/layout.xml", mimeType = "application/octet-stream")]
		public static var layout_xml : Class;
		
		[Embed(source = "../../res/scene_3/chair.png")]
		public static var chair_png : Class;
		
		[Embed(source = "../../res/scene_3/desk.png")]
		public static var desk_png : Class;
		
		[Embed(source = "../../res/scene_3/desk_collision.png")]
		public static var desk_collision_png : Class;
		
		[Embed(source = "../../res/scene_3/chair_collision.png")]
		public static var chair_collision_png : Class;
		
		private var _body : Body;
		private var _desk : FlxSprite;
		private var _chair : FlxSprite;
		private var _cursor : Cursor;
		
		private var _grabables : FlxGroup;
		private var _chairCollision : FlxSprite;
		private var _deskCollision : FlxSprite;
		
		private var _world : b2World;
		
		private var _nextRight : Boolean = false;
		private var _targetRight : Point;
		private var _targetLeft : Point;
		
		private var _biy : Number = 0;
		
		public function Scene3State() 
		{
			FlxPhysBody.WORLD_SCALE = 30;
			var grav : b2Vec2 = new b2Vec2(0, -30);
			//var grav : b2Vec2 = new b2Vec2(0, 0);
			
			_world = new b2World(grav, false);
			
			_desk = new FlxSprite(0, 0, desk_png);
			_desk.offset.x = _desk.width / 2;
			_desk.offset.y = _desk.height / 2;
			
			_chair = new FlxSprite(0, 0, chair_png);
			_chair.offset.x = _chair.width / 2;
			_chair.offset.y = _chair.height / 2;
			
			_grabables = new FlxGroup();
			_grabables.visible = false;
			
			//Floor
			
			//var _floor : Ground = new Ground(FlxG.width / 2, FlxG.height + 20, FlxG.width * 20, 40, _world);
			//add(_floor);
			
			_cursor = new Cursor();
		}
		
		override public function create():void
		{
			loadXML(new XMLLoader(new XML(new layout_xml())));
		}
		
		override public function update():void
		{
			_world.Step(FlxG.elapsed * 2, 10, 10);
			_world.ClearForces();
			
			if (_targetLeft != null)
			{
				var dx : Number = _targetLeft.x - _body.hand_l.x;
				var dy : Number = _targetLeft.y - _body.hand_l.y;
				
				//dx *= .9 * FlxG.elapsed;
				//dy *= .9 * FlxG.elapsed;
				
				
				
				var tx : Number = _body.hand_l.x + dx;
				var ty : Number = _body.hand_l.y + dy;
				
				//_body.hand_l.setPositionPixels(tx, ty);
				_body.hand_l.body.ApplyImpulse(new b2Vec2(dx, dy * -1), _body.hand_l.body.GetPosition());
			}
			
			if (_targetRight != null)
			{
				dx = _targetRight.x - _body.hand.x;
				dy = _targetRight.y - _body.hand.y;
				
				//dx *= .9 * FlxG.elapsed;
				//dy *= .9 * FlxG.elapsed;
				
				//dx *= 1;
				//dy *= 1;
				
				tx = _body.hand.x + dx;
				ty = _body.hand.y + dy;
				
				//_body.hand.setPositionPixels(tx, ty);
				_body.hand.body.ApplyImpulse(new b2Vec2(dx, dy * -1), _body.hand.body.GetPosition());
			}
			
			doHands();
			
			if (FlxG.mouse.justPressed())
			{
				doClick();
			}
			
			//Exit
			
			if (_body.body.y < 0)
			{
				FlxG.fade.start(0xFF000000, 5, finish);
			}
			
			super.update();
		}
		
		private function finish():void
		{
			FlxG.state = new EndState();
		}
		
		private function doHands():void
		{
			var u : Boolean = false;
			
			var hx : Number = _body.hand.x;
			var hy : Number = _body.hand.y;
			
			if (_chairCollision.overlapsPoint(hx, hy) || _deskCollision.overlapsPoint(hx, hy))
			{
				u = true;
			}
			
			hx = _body.hand_l.x;
			hy = _body.hand_l.y;
			
			if ((_chairCollision.overlapsPoint(hx, hy) || _deskCollision.overlapsPoint(hx, hy)) && u)
			{
				u = true;
			} else {
				u = false;
			}
			
			if (u)
			{
				var ny : Number = _body.body.y - FlxG.elapsed * 10;
				_body.body.setPositionPixels(_body.body.x, ny);
				
				var da : Number = _body.body.body.GetAngle();
				da -= FlxG.elapsed / 10 * Math.PI / 2;
				
				if (da < 0)
				{
					da = 0;
				}
				
				_body.body.body.SetAngle(da);
			} else {
				ny = _body.body.y + FlxG.elapsed * 50;
				if (ny > _biy) ny = _biy;
				_body.body.setPositionPixels(_body.body.x, ny);
				
				if (ny == _biy)
				{
					da = _body.body.body.GetAngle();
					da += FlxG.elapsed / 10 * Math.PI / 2;
					
					if (da > Math.PI / 2)
					{
						da = Math.PI / 2;
					}
					
					_body.body.body.SetAngle(da);
				}
			}
		}
		
		private function doClick():void
		{
			var p : Point = new Point();
			p.x = FlxG.mouse.x;
			p.y = FlxG.mouse.y;
			
			var dlx : Number = _body.hand_l.x - p.x;
			var dly : Number = _body.hand_l.y - p.y;
			var dl : Number = dlx * dlx + dly * dly;
			
			var drx : Number = _body.hand.x - p.x;
			var dry : Number = _body.hand.y - p.y;
			var dr : Number = drx * drx + dry * dry;
			
			if (dr < dl)
			{
				_targetRight = p;
				
				if (_body.hand.body.GetType() == b2Body.b2_kinematicBody)
				{
					_body.hand.body.SetType(b2Body.b2_dynamicBody);
				}
			} else {
				_targetLeft = p;
				
				if (_body.hand_l.body.GetType() == b2Body.b2_kinematicBody)
				{
					_body.hand_l.body.SetType(b2Body.b2_dynamicBody);
				}
			}
			
			_nextRight = !_nextRight;
		}
		
		private function loadXML(xml : XMLLoader):void
		{
			var d : Object = xml.data;
			_body = new Body(_world, d.body.x, d.body.y, d.body.rotation);
			_biy = d.body.y;
			
			_body.hand.body.SetType(b2Body.b2_kinematicBody);
			_body.hand_l.body.SetType(b2Body.b2_kinematicBody);
			_body.body.body.SetType(b2Body.b2_kinematicBody);
			
			_body.hand.setPositionPixels(d.hand_r.x, d.hand_r.y);
			_body.hand_l.setPositionPixels(d.hand_l.x, d.hand_l.y);
			_body.hand.body.SetAngle(d.hand_r.rotation * -1);
			_body.hand_l.body.SetAngle(d.hand_l.rotation * -1);
			
			_world.Step(10, 10, 10);
			_world.ClearForces();
			
			//_body.hand.body.SetType(b2Body.b2_dynamicBody);
			//_body.hand_l.body.SetType(b2Body.b2_dynamicBody);
			//_body.body.body.SetType(b2Body.b2_dynamicBody);
			
			/*for (var t : Number = 0; t < 30; t += .1)
			{
				_world.Step(.1, 10, 10);
				_world.ClearForces();
			}*/
			
			
			_desk.x = d.desk.x;
			_desk.y = d.desk.y;
			
			_chair.x = d.chair.x;
			_chair.y = d.chair.y;
			
			var _dc : FlxSprite = new FlxSprite(d.desk_collision.x, d.desk_collision.y, desk_collision_png);
			var _cc : FlxSprite = new FlxSprite(d.chair_collision.x, d.chair_collision.y, chair_collision_png);
			
			_dc.offset.x = _dc.width / 2;
			_dc.offset.y = _dc.height / 2;
			_cc.offset.x = _cc.width / 2;
			_cc.offset.y = _cc.height / 2;
			
			_grabables.add(_dc);
			_grabables.add(_cc);
			
			_deskCollision = _dc;
			_chairCollision = _cc;
			
			add(_desk);
			add(_chair);
			add(_grabables);
			add(_body);
			
			add(_cursor);
		}
		
	}

}